package test;

import org.jbpm.JbpmConfiguration;
import org.jbpm.JbpmContext;
import org.jbpm.db.GraphSession;
import org.jbpm.graph.def.Action;
import org.jbpm.graph.def.Node;
import org.jbpm.graph.def.ProcessDefinition;
import org.jbpm.graph.def.Transition;
import org.jbpm.graph.exe.ProcessInstance;
import org.jbpm.graph.exe.Token;
import org.jbpm.graph.node.EndState;
import org.jbpm.graph.node.StartState;
import org.jbpm.graph.node.State;
import org.jbpm.instantiation.Delegation;
import org.jbpm.job.executor.JobExecutor;

public class TestCase {
	private static JbpmConfiguration jbpmConfiguration = JbpmConfiguration.getInstance();
	private static JobExecutor jobExecutor;
	
	public static void main(String[] args) {
		TestCase.setUp();
		final TestCase testCase = new TestCase();
		testCase.test1();
		TestCase.tearDown();
	}
	
	public static void setUp() {
		final JbpmContext jbpmContext = jbpmConfiguration.createJbpmContext();
		try {
			// process definition
			final StartState startState = new StartState("start");
			final Node node1 = new Node("node1");
			final State state1 = new State("state1");
			final EndState endState = new EndState("end");
			final Transition transition1 = new Transition();
			final Transition transition2 = new Transition();
			final Transition transition3 = new Transition();
			final Action action1 = new Action(new Delegation("test.LongRunningNode"));
			
			node1.setAction(action1);
			node1.setAsync(true);
			// 1: start -> node1
			startState.addLeavingTransition(transition1);
			node1.addArrivingTransition(transition1);
			// 2: node1 -> state1
			node1.addLeavingTransition(transition2);
			state1.addArrivingTransition(transition2);
			// 3: state1 -> end
			state1.addLeavingTransition(transition3);
			endState.addArrivingTransition(transition3);
			
			final ProcessDefinition processDefinition = ProcessDefinition.createNewProcessDefinition();
			processDefinition.setName("process");
			
			processDefinition.addNode(startState);
			processDefinition.addNode(node1);
			processDefinition.addNode(state1);
			processDefinition.addNode(endState);
			
			jbpmConfiguration.createSchema();
			jbpmContext.deployProcessDefinition(processDefinition);
		} finally {
			jbpmContext.close();
		}
		jobExecutor = jbpmConfiguration.getJobExecutor();
		jobExecutor.start();
		
	}
	
	public void test1() {
		final Object monitor = new Object();
		LongRunningNodeSynchronization.setMonitor(monitor);
		
		long processInstanceId = 0;
		JbpmContext jbpmContext = jbpmConfiguration.createJbpmContext();
		try {
			final GraphSession graphSession = jbpmContext.getGraphSession();
			final ProcessDefinition processDefinition = graphSession.findLatestProcessDefinition("process");
			final ProcessInstance processInstance = new ProcessInstance(processDefinition);
			processInstanceId = processInstance.getId();
			final Token token = processInstance.getRootToken();
			
			// send the token to the async node
			token.signal();
			
			jbpmContext.save(processInstance);
		} finally {
			jbpmContext.close();
		}
		
		// the async node should now be executed
		{
			boolean done = false;
			while (!done) {
				synchronized (monitor) {
					try {
						monitor.wait();
					} catch (final InterruptedException e) {
						continue;
					}
				}
				done = true;
			}
		}
		// now, it should be finished

		jbpmContext = jbpmConfiguration.createJbpmContext();
		try {
			final GraphSession graphSession = jbpmContext.getGraphSession();
			final ProcessInstance processInstance = graphSession.getProcessInstance(processInstanceId);
			final Token token = processInstance.getRootToken();

			// check for correct node
			if (!"state1".equals(token.getNode().getName())) {
				System.err.println("Expected \"state1\" but was \"" + token.getNode().getName() + "\"");
				return;
			} else {
				System.out.println("Token is now in node \"state1\" as expected");
			}
			
			token.signal();

			// check for correct node
			if (!"end".equals(token.getNode().getName())) {
				System.err.println("Expected \"end\" but was \"" + token.getNode().getName() + "\"");
				return;
			} else {
				System.out.println("Token is now in node \"end\" as expected");
			}
			// check if process instance has ended
			if (!processInstance.hasEnded()) {
				System.err.println("Process instance has not ended yet");
				return;
			} else {
				System.out.println("Process instance has ended");
			}
			// check if token has ended
			if (!token.hasEnded()) {
				System.err.println("Token has not ended yet");
				return;
			} else {
				System.out.println("Token has ended");
			}
			
			jbpmContext.save(processInstance);
		} finally {
			jbpmContext.close();
		}
	}
	
	public static void tearDown() {
		{
			boolean done = false;
			while (!done) {
				try {
					jobExecutor.stopAndJoin();
				} catch (final InterruptedException e1) {
					continue;
				}
				done = true;
			}
		}
	}
}
